# This file is part of fedmsg.
# Copyright (C) 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  David Gay <oddshocks@riseup.net

from fedmsg_meta_fedora_infrastructure import BaseProcessor


class FedimgProcessor(BaseProcessor):
    __name__ = "fedimg"
    __description__ = "The Fedora cloud image service"
    __link__ = "https://github.com/oddshocks/fedimg"
    __docs__ = "https://fedoraproject.org/wiki/Features/" + \
               "FirstClassCloudImages/KojiPlan"
    __obj__ = "New cloud image upload"
    __icon__ = "https://apps.fedoraproject.org/img/icons/fedimg.png"

    def subtitle(self, msg, **config):
        name = msg['msg']['image_name']
        dest = msg['msg']['destination']
        if 'image.upload' in msg['topic']:
            if msg['msg']['status'] == "started":
                tmpl = self._('{image_name} started uploading to {dest}')
                return tmpl.format(image_name=name, dest=dest)
            elif msg['msg']['status'] == "completed":
                ami = msg['msg']['extra']['id']
                virt = msg['msg']['extra']['virt_type']
                vol = msg['msg']['extra']['vol_type']
                tmpl = self._('{image_name} finished uploading to {dest} '
                              '({ami}, {virt}, {vol})')
                return tmpl.format(image_name=name, dest=dest, ami=ami,
                                   virt=virt, vol=vol)
            elif msg['msg']['status'] == "failed":
                tmpl = self._('{image_name} failed to upload to {dest}')
                return tmpl.format(image_name=name, dest=dest)
        if 'image.test' in msg['topic']:
            tmpl = ''
            ami = msg['msg']['extra']['id']
            virt = msg['msg']['extra']['virt_type']
            vol = msg['msg']['extra']['vol_type']
            if msg['msg']['status'] == "started":
                extra = msg['msg']['extra']
                tmpl = self._('{image_name} started testing on {dest} '
                              '({ami}, {virt}, {vol})')
            elif msg['msg']['status'] == "completed":
                tmpl = self._('{image_name} finished testing on {dest} '
                              '({ami}, {virt}, {vol})')
            elif msg['msg']['status'] == "failed":
                tmpl = self._('{image_name} failed testing on {dest} '
                              '({ami}, {virt}, {vol})')
            return tmpl.format(image_name=name, dest=dest, ami=ami,
                               virt=virt, vol=vol)

    def objects(self, msg, **config):
        status = msg['msg']['status']
        if 'image.upload' in msg['topic']:
            return set(['image/upload/' + status])
        elif 'image.test' in msg['topic']:
            return set(['image/test/' + status])



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for that one datanommer message """

import unittest

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestDatanommerWat(Base):
    """ Generally speaking, the `datanommer
    <https://github.com/fedora-infra/datanommer>`_ consumer does not publish
    its own fedmsg messages.  There is one exception to this rule.

    As of fedmsg-0.7.0, every fedmsg carries its own uuid, prefixed with the
    year it was published.   There is a very low risk of creating a duplicate
    uuid.  To quote wikipedia::

        "The annual risk of someone being hit by a meteorite is estimated to be
        one chance in 17 billion, which means the probability is about
        ``0.00000000006 (6 * 10**-11)``, equivalent to the odds of creating a
        few tens of trillions of UUIDs in a year and having one duplicate"

    It is highly unlikely that datanommer will ever try to store a fedmsg
    message that carries a uuid that already exists in its database.  In the
    event that it does, it will publish the following message; a momentous
    occasion.
    """

    expected_title = "datanommer.wat"
    expected_subti = 'datanommer encountered a duplicate uuid'
    expected_link = "https://www.destroyallsoftware.com/talks/wat"
    expected_icon = "https://i.imgur.com/4g9NZu1.png"
    expected_secondary_icon = "https://i.imgur.com/58oJkOr.gif"
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['wat'])
    msg = {
        "username": "fedmsg",
        "i": 1,
        "timestamp": 1375753735.32427,
        "topic": "org.fedoraproject.prod.datanommer.wat",
        "msg": {
            "uuid": "2013-3bf0ec8f-03d3-40be-9ad5-5effdc6e4c06",
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Michael Simacek <msimacek@redhat.com>

from fedmsg_meta_fedora_infrastructure import BaseProcessor

class KoscheiProcessor(BaseProcessor):
    __name__ = 'koschei'
    __description__ = "Continuous integration for Fedora packages"
    __link__ = "http://koschei.cloud.fedoraproject.org"
    __docs__ = "http://koschei.cloud.fedoraproject.org/documentation"
    __obj__ = "Watched packages"
    __icon__ = ("https://fedoraproject.org/w/uploads/e/e9/Koschei.png")

    def subtitle(self, msg, **config):
        if 'koschei.package.state.change' in msg['topic']:
            content = msg['msg']
            if content['new'] == 'ok' and content['old'] == 'ignored':
                info = "{name} added to Koschei"
            else:
                info = {'failing': "{name}'s builds started to fail",
                        'ok': "{name}'s builds are back to normal",
                        'ignored': "{name} became retired or ignored",
                        'unresolved': "{name}'s dependencies failed to resolve",
                        }[content['new']]
            info += ' in {repo}'
            if content['koji_instance'] != 'primary':
                info += ' ({koji_instance})'
            return info.format(name=content['name'], repo=content['repo'],
                               koji_instance=content['koji_instance'])
        else:
            raise NotImplementedError("%r" % msg)

    def secondary_icon(self, msg, **config):
        tmpl = 'https://apps.fedoraproject.org/packages/images/icons/%s.png'
        if 'koschei.package.state.change' in msg['topic']:
            return tmpl % msg['msg']['name']

    def link(self, msg, **config):
        baseurl = 'http://koschei.cloud.fedoraproject.org'
        if 'koschei.package.state.change' in msg['topic']:
            return '{baseurl}/package/{name}'.format(baseurl=baseurl,
                                                     name=msg['msg']['name'])
        else:
            raise NotImplementedError("%r" % msg)

    def packages(self, msg, **config):
        if 'koschei.package.state.change' in msg['topic']:
            return set([msg['msg']['name']])
        else:
            raise NotImplementedError("%r" % msg)



import fedmsg.meta.base
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url


class ByUserAndPackageTesting(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return 'bodhi.update.request.testing' in msg['topic']

    def matches(self, a, b, **config):
        """ The messages must match with both user and package """
        if a['msg']['agent'] != b['msg']['agent']:
            return False
        package_a = self.processor._u2p(a['msg']['update']['title'])[0]
        package_b = self.processor._u2p(b['msg']['update']['title'])[0]
        if package_a != package_b:
            return False
        return True

    def merge(self, constituents, **config):
        N = len(constituents)
        msg = constituents[0]['msg']
        agent = msg['agent']
        package = self.processor._u2p(msg['update']['title'])[0]
        branches = self.list_to_series([
            constituent['msg']['update']['release']['name']
            for constituent in constituents])

        tmpl = self.produce_template(constituents, **config)
        subtitle = '{agent} submitted {N} {package} ' + \
            'testing updates for {branches}'
        tmpl['subtitle'] = subtitle.format(
            agent=agent, package=package, N=N, branches=branches)
        tmpl['secondary_icon'] = avatar_url(msg['agent'])
        base = 'https://admin.fedoraproject.org/updates/%s/'
        tmpl['link'] = base % package
        return tmpl


class ByUserAndPackageStable(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return 'bodhi.update.request.stable' in msg['topic']

    def matches(self, a, b, **config):
        """ The messages must match with both user and package """
        if a['msg']['agent'] != b['msg']['agent']:
            return False
        package_a = self.processor._u2p(a['msg']['update']['title'])[0]
        package_b = self.processor._u2p(b['msg']['update']['title'])[0]
        if package_a != package_b:
            return False
        return True

    def merge(self, constituents, **config):
        N = len(constituents)
        msg = constituents[0]['msg']
        agent = msg['agent']
        package = self.processor._u2p(msg['update']['title'])[0]
        branches = self.list_to_series([
            constituent['msg']['update']['release']['name']
            for constituent in constituents])

        tmpl = self.produce_template(constituents, **config)
        subtitle = '{agent} requested {N} {package} ' + \
            'stable updates for {branches}'
        tmpl['subtitle'] = subtitle.format(
            agent=agent, package=package, N=N, branches=branches)
        tmpl['secondary_icon'] = avatar_url(msg['agent'])
        base = 'https://admin.fedoraproject.org/updates/%s/'
        tmpl['link'] = base % package
        return tmpl


class ByPackage(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return any([
            'bodhi.update.request.testing' in msg['topic'],
            'bodhi.update.request.stable' in msg['topic'],
        ])

    def matches(self, a, b, **config):
        """ The message must match by package """
        package_a = self.processor._u2p(a['msg']['update']['title'])[0]
        package_b = self.processor._u2p(b['msg']['update']['title'])[0]
        if package_a != package_b:
            return False
        return True

    def merge(self, constituents, **config):
        N = len(constituents)
        msg = constituents[0]['msg']
        package = self.processor._u2p(msg['update']['title'])[0]
        branches = self.list_to_series([
            constituent['msg']['update']['release']['name']
            for constituent in constituents])

        tmpl = self.produce_template(constituents, **config)
        agents = self.list_to_series(list(tmpl['usernames']))
        subtitle = '{agents} submitted {N} {package} ' + \
            'updates for {branches}'
        tmpl['subtitle'] = subtitle.format(
            agents=agents, package=package, N=N, branches=branches)
        tmpl['secondary_icon'] = tmpl['icon']
        base = 'https://admin.fedoraproject.org/updates/%s/'
        tmpl['link'] = base % package
        return tmpl


class ByUser(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return any([
            'bodhi.update.request.testing' in msg['topic'],
            'bodhi.update.request.stable' in msg['topic'],
        ])

    def matches(self, a, b, **config):
        """ The message must match by username """
        if a['msg']['agent'] != b['msg']['agent']:
            return False
        return True

    def merge(self, constituents, **config):
        N = len(constituents)
        msg = constituents[0]['msg']
        agent = msg['agent']
        branches = self.list_to_series([
            constituent['msg']['update']['release']['name']
            for constituent in constituents])

        tmpl = self.produce_template(constituents, **config)
        packages = self.list_to_series(list(tmpl['packages']))
        subtitle = '{agent} submitted {packages} ' + \
            'updates for {branches}'
        tmpl['subtitle'] = subtitle.format(
            agent=agent, packages=packages, N=N, branches=branches)
        tmpl['secondary_icon'] = avatar_url(msg['agent'])
        base = 'https://admin.fedoraproject.org/updates/user/%s/'
        tmpl['link'] = base % agent
        return tmpl
